"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const SERVER_CLOSE_TIMEOUT = 100;
function App() {
    let app;
    return {
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                return app = express();
            });
        }
    };
}
exports.App = App;
function Server() {
    let server;
    let serverIsOpen = false;
    return {
        start({ config, app }, _, stop) {
            return new Promise((resolve, reject) => {
                if (!config.port)
                    return reject(new Error(`Missing port config`));
                server = app.listen(Number(config.port), config.hostname || '0.0.0.0', err => {
                    if (err)
                        return reject(err);
                    serverIsOpen = true;
                    resolve(server);
                })
                    .on('error', stop)
                    .on('close', () => { serverIsOpen = false; });
            });
        },
        stop() {
            return new Promise((resolve, reject) => {
                if (!server || !serverIsOpen)
                    return resolve();
                server.close(err => {
                    if (err)
                        return reject(err);
                    resolve();
                });
                setTimeout(() => reject(new Error("Express server close timeout")), SERVER_CLOSE_TIMEOUT);
            });
        }
    };
}
exports.Server = Server;
