import System from 'corpjs-system'
import http = require('http')
import express = require('express')

const SERVER_CLOSE_TIMEOUT = 100

export function App(): System.Component {

  let app: express.Application

  return {
    async start(): Promise<express.Application> {
      return app = express()
    }
  }

}

export interface ServerConfig {
  port: number | string
  hostname?: string
}

export interface ServerDeps {
  app: express.Application,
  config: ServerConfig
}

export function Server(): System.Component {

  let server: http.Server
  let serverIsOpen: boolean = false

  return {

    start({ config, app }: ServerDeps, _, stop): Promise<http.Server> {
      return new Promise<http.Server>((resolve, reject) => {
        if (!config.port) return reject(new Error(`Missing port config`))
        server = app.listen(Number(config.port), config.hostname || '0.0.0.0', err => {
          if (err) return reject(err)
          serverIsOpen = true
          resolve(server)
        })
          .on('error', stop)
          .on('close', () => { serverIsOpen = false })
      })
    },

    stop(): Promise<void> {
      return new Promise<void>((resolve, reject) => {
        if (!server || !serverIsOpen) return resolve()
        server.close(err => {
          if (err) return reject(err)
          resolve()
        })
        setTimeout(
          () => reject(new Error("Express server close timeout")),
          SERVER_CLOSE_TIMEOUT
        )
      })
    }
  }

}